﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Arction.Wpf.SemibindableCharting;
using Arction.Wpf.SemibindableCharting.Views;
using Arction.Wpf.SemibindableCharting.Views.View3D;
using Arction.Wpf.SemibindableCharting.Series3D; 

namespace WpfMeshValueBased
{
    /// <summary>
    /// Example how to make SurfaceMeshSeries with geometry of a pipe, and coloring it by values 
    /// Copyright 2016 Arction Ltd. 
    /// Usage only in LightningChart related code. 
    public partial class MainWindow : Window
    {

        const int Columns = 150;
        const int Rows = 150;

        public MainWindow()
        {
            InitializeComponent();
            

            double[,] x = null;
            double[,] y = null;
            double[,] z = null;
            double[,] values = null;

            CreateGeometryPipe(out x, out y, out z);

            CreateValuesWaterDrops(out values);

            UpdateMesh(x, y, z, values);

        }

       
        void UpdateMesh(double[,] x, double[,] y, double[,] z, double[,] values)
        {


            int iCols = x.GetLength(0);
            int iRows = x.GetLength(1);
            SurfacePoint[,] data = new SurfacePoint[iCols, iRows];

            for (int iCol = 0; iCol < iCols; iCol++)
            {
                for (int iRow = 0; iRow < iRows; iRow++)
                {
                    data[iCol, iRow].X = x[iCol, iRow];
                    data[iCol, iRow].Y = y[iCol, iRow];
                    data[iCol, iRow].Z = z[iCol, iRow];
                    data[iCol, iRow].Value = values[iCol, iRow];
                }

            }

            _mesh.Data = data;
        }




        //Pipe
        void CreateGeometryPipe(out double[,] x, out double[,] y, out double[,] z)
        {
            x = new double[Columns, Rows];
            y = new double[Columns, Rows];
            z = new double[Columns, Rows];
            double dXMin = 0;
            double dXMax = 100;
            double dYMin = 0;
            double dYMax = 100;
            double dZMin = 0;
            double dZMax = 100;
            double dCenterX = (dXMin + dXMax) / 2.0;
            double dCenterZ = (dZMin + dZMax) / 2.0;
            double dAngle;
            double dYStep = (dYMax - dYMin) / (double)(Rows - 1);
            double dAngleStep = 2.0 * Math.PI / (double)(Columns - 1); //round tube 
            double dXRadius = 40;
            double dZRadius = 40;

            for (int iCol = 0; iCol < Columns; iCol++)
            {
                dAngle = (double)iCol * dAngleStep;
                for (int iRow = 0; iRow < Rows; iRow++)
                {

                    x[iCol, iRow] = dCenterX + dXRadius * Math.Sin(dAngle);
                    y[iCol, iRow] = dYMin + (double)iRow * dYStep;
                    z[iCol, iRow] = dCenterZ + dZRadius * Math.Cos(dAngle);
                }
            }
        }


        void CreateValuesWaterDrops(out double[,] values)
        {
            values = WaterDropSurfaceDataGenerator.Generate(
                Columns, //size as nodes, in both X and Y
                new double[] { 0.5, 0.5, 0.7, 0.05 }, //Drop X positions in scale 0...1
                new double[] { 0.2, 0.9, 0.7, 0.95 }, //Drop Z positions in scale 0...1
                new double[] { 30, 50, 15, 50 }, //Amplitudes, as Y axis values
                40, //Offset level (mid-Y) 
                15); //Volatility, wave generating density

        }

    }

    
}
